import { ItemStack, ItemTypes, world } from "@minecraft/server"
import { ModalFormData } from "@minecraft/server-ui"

const Menu = {
    blockTypeId: "sra:scp_294",
    itemTypeId: "minecraft:iron_nugget"
}

const Drinks = [
    {
        Display: "Apple Juice",
        Input: ["Apple Juice", "Apple"],
        Give: {
            ItemType: "sra:cup_of_apple_juice",
            Amount: 1
        },
        Cost: {
            ItemType: "minecraft:iron_nugget",
            Amount: 3
        }
    },
    {
        Display: "Bleach",
        Input: ["Bleach"],
        Give: {
            ItemType: "sra:cup_of_bleach",
            Amount: 1
        },
        Cost: {
            ItemType: "minecraft:iron_nugget",
            Amount: 3
        }
    },
    {
        Display: "Blood",
        Input: ["Blood"],
        Give: {
            ItemType: "sra:cup_of_blood",
            Amount: 1
        },
        Cost: {
            ItemType: "minecraft:iron_nugget",
            Amount: 3
        }
    },
    {
        Display: "Coffee",
        Input: ["Coffee"],
        Give: {
            ItemType: "sra:cup_of_coffee",
            Amount: 1
        },
        Cost: {
            ItemType: "minecraft:iron_nugget",
            Amount: 5
        }
    },
    {
        Display: "Cola",
        Input: ["Cola", "Coke"],
        Give: {
            ItemType: "sra:cup_of_cola",
            Amount: 1
        },
        Cost: {
            ItemType: "minecraft:iron_nugget",
            Amount: 3
        }
    },
    {
        Display: "Enchanted Golden Apple",
        Input: ["Enchanted Golden Apple"],
        Give: {
            ItemType: "sra:cup_of_enchanted_golden_apple",
            Amount: 1
        },
        Cost: {
            ItemType: "minecraft:iron_nugget",
            Amount: 45
        }
    },
    {
        Display: "Golden Apple",
        Input: ["Golden", "Golden Apple"],
        Give: {
            ItemType: "sra:cup_of_golden_apple",
            Amount: 1
        },
        Cost: {
            ItemType: "minecraft:iron_nugget",
            Amount: 30
        }
    },
    {
        Display: "Hydrochloric Acid",
        Input: ["Hydrochloric Acid", "Hydrochloric", "Acid"],
        Give: {
            ItemType: "sra:cup_of_hydrochloric_acid",
            Amount: 1
        },
        Cost: {
            ItemType: "minecraft:iron_nugget",
            Amount: 3
        }
    },
    {
        Display: "Lava",
        Input: ["Lava"],
        Give: {
            ItemType: "sra:cup_of_lava",
            Amount: 1
        },
        Cost: {
            ItemType: "minecraft:iron_nugget",
            Amount: 3
        }
    },
    {
        Display: "Lemonade",
        Input: ["Lemonade", "Lemon"],
        Give: {
            ItemType: "sra:cup_of_lemonade",
            Amount: 1
        },
        Cost: {
            ItemType: "minecraft:iron_nugget",
            Amount: 3
        }
    },
    {
        Display: "Milk",
        Input: ["Milk"],
        Give: {
            ItemType: "sra:cup_of_milk",
            Amount: 1
        },
        Cost: {
            ItemType: "minecraft:iron_nugget",
            Amount: 10
        }
    },
    {
        Display: "Orange Juice",
        Input: ["Orange", "Orange Juice"],
        Give: {
            ItemType: "sra:cup_of_orange_juice",
            Amount: 1
        },
        Cost: {
            ItemType: "minecraft:iron_nugget",
            Amount: 3
        }
    },
    {
        Display: "SCP-075 Corrosion",
        Input: ["SCP-075", "SCP-075 Corrosion", "Corrosion", "SCP 075", "075"],
        Give: {
            ItemType: "sra:cup_of_scp_075_corrosion",
            Amount: 1
        },
        Cost: {
            ItemType: "minecraft:iron_nugget",
            Amount: 5
        }
    },
    {
        Display: "Water",
        Input: ["Water"],
        Give: {
            ItemType: "sra:cup_of_water",
            Amount: 1
        },
        Cost: {
            ItemType: "minecraft:iron_nugget",
            Amount: 3
        }
    }
]
world.afterEvents.entityHitBlock.subscribe((e) => {
    const { hitBlock, damagingEntity, blockFace } = e
    try {
        if (hitBlock.typeId === Menu.blockTypeId) {
            let item = damagingEntity.getComponent("minecraft:inventory").container.getItem(damagingEntity.selectedSlot)
            if (item.typeId === Menu.itemTypeId)
                return OpenMenu(damagingEntity)
        }
    } catch (e) {
        console.warn(e)
    }
})

//Opens the menu for the player
function OpenMenu(player, error = undefined) {
    const form = new ModalFormData()
        .title(error ?? `SCP-294`)
        .textField(``, `                  Drink Name`, ``)
    form.show(player).then((data) => {
        try {
            if (!data)
                return;
            let drink = FindDrink(data.formValues[0].toLowerCase())
            let playersCurrency = GetItemAmount(player.getComponent("minecraft:inventory").container, drink.Cost.ItemType)
            if (playersCurrency < drink.Cost.Amount)
                return OpenMenu(player, `§4Insufficient Iron Nuggets!`)
            return GiveItem(player, drink)
        } catch (e) {
            console.warn(e)
        }
    })
}

function FindDrink(drinkName) {
    for (let i = 0; i < Drinks.length; i++) {
        let Elements = Drinks[i].Input.map(e => e.toLowerCase())
        if (Elements.includes(drinkName))
            return Drinks[i]
    }
    return undefined
}

/**
 * @description This function will return the amount of an item type in a container
 * @param container Container to search in
 * @param itemTypeId Type Id of the item
*/
function GetItemAmount(container, itemTypeId) {
    let amount = 0;
    for (let i = 0; i < container.size; i++) {
        let item = container.getItem(i)
        if (!item)
            continue
        if (item.typeId !== itemTypeId)
            continue
        amount += item.amount
    }
    return amount
}


function GiveItem(player, drinkData) {
    try {

        let container = player.getComponent("minecraft:inventory").container
        let itemStack = new ItemStack(ItemTypes.get(drinkData.Give.ItemType), drinkData.Give.Amount)
        if (container.emptySlotsCount > 0) {
            container.addItem(itemStack)
        } else {
            world.getDimension("overworld").spawnItem(itemStack, player.location)
        }
        player.runCommandAsync(`clear @s ${drinkData.Cost.ItemType} -1 ${drinkData.Cost.Amount}`)

    } catch (e) {
        console.warn(e)
    }
}